function [K, dK, M] = ex10_aux_func(dim, T, ell, alpha, ...
                                    num_filter, invalid_pts, tag)
%function [K, dK, M] = ex10_aux_func(dim, T, ell, alpha, ...
%                                    num_filter, invalid_pts, tag)
%
% This function generates the covariance matrix K, partial derivatives
% dK, and preconditioner M needed by ex10.m. The forms of the outputs
% depend on the tag, which can be either 'mat' or 'func'.
%

%Author: J. Chen
%$Date: 2011/12/20 12:51:48$

% generate covariance matrix for regular grid based on current ell
[tK, tdK] = gen_K_power_regular(dim, T, ell, alpha);

% filter K using Laplace filter; perform K mult by circulant embedding
tK = lapFilterMat2d(tK, num_filter);
if strcmp(tag, 'func')
  [~, eigvalC] = embed(tK);
  K = @(x)aux_fun(eigvalC, x, invalid_pts);
elseif strcmp(tag, 'mat')
  K = toe2mat(tK);
  K(invalid_pts,:) = [];
  K(:,invalid_pts) = [];
else
  error('wrong tag argument');
end

% derivatives of K
for i = 1:length(tdK)
  tdK{i} = lapFilterMat2d(tdK{i}, num_filter);
  if strcmp(tag, 'func')
    [~, eigvaldC{i}] = embed(tdK{i});
    dK{i} = @(x)aux_fun(eigvaldC{i}, x, invalid_pts);
  elseif strcmp(tag, 'mat')
    dK{i} = toe2mat(tdK{i});
    dK{i}(invalid_pts,:) = [];
    dK{i}(:,invalid_pts) = [];
  else
    error('wrong tag argument');
  end
end

% no preconditioner
M = [];


function [y] = aux_fun(eigvalC, x, invalid_pts)

% pad x to xx due to invalid points
len = prod(size(eigvalC)/2);
idx = 1:len;
idx(invalid_pts) = [];
xx = zeros(len,size(x,2));
xx(idx,:) = x;

% the multiplication by circulant embedding
y = toeplitz_mult(eigvalC, xx);

% trim unused portion due to invalid points
y(invalid_pts,:) = [];

