function [K, dK, M] = ex3_aux_func(dim, T, ell, nu, num_filter, tag)
%function [K, dK, M] = ex3_aux_func(dim, T, ell, nu, num_filter, tag)
%
% This function generates the covariance matrix K, partial derivatives
% dK, and preconditioner M needed by ex3.m. The forms of the outputs
% depend on the tag, which can be either 'mat' or 'func'.
%

%Author: J. Chen
%$Date: 2011/12/20 12:50:50$

% generate covariance matrix for regular grid based on current ell
[tK, tdK] = gen_K_matern_regular(dim, T, ell, nu);

% filter K using Laplace filter; perform K mult by circulant embedding
tK = lapFilterMat3d(tK, num_filter);
if strcmp(tag, 'func')
  [~, eigvalC] = embed(tK);
  K = @(x)toeplitz_mult(eigvalC, x);
elseif strcmp(tag, 'mat')
  K = toe2mat(tK);
else
  error('invalid tag argument');
end

% derivatives of K
for i = 1:length(tdK)
  tdK{i} = lapFilterMat3d(tdK{i}, num_filter);
  if strcmp(tag, 'func')
    [~, eigvaldC{i}] = embed(tdK{i});
    dK{i} = @(x)toeplitz_mult(eigvaldC{i}, x);
  elseif strcmp(tag, 'mat')
    dK{i} = toe2mat(tdK{i});
  else
    error('invalid tag argument');
  end
end

% use circulant preconditioner
if strcmp(tag, 'func')
  eigvalM = 1./circ_precond(tK);
  M = @(x)circ_mult(eigvalM, x);
elseif strcmp(tag, 'mat')
  M = [];
else
  error('invalid tag argument');
end
