function [K, dK, M] = ex4_aux_func(coord, ell, nu, L, tag)
%function [K, dK, M] = ex4_aux_func(coord, ell, nu, L, tag)
%
% This function generates the covariance matrix K, partial derivatives
% dK, and preconditioner M needed by ex4.m. The forms of the outputs
% depend on the tag, which can be either 'mat' or 'func'.
%

%Author: J. Chen
%$Date: 2011/12/20 12:50:57$

% generate covariance matrix for random points based on current ell
[K, dK] = gen_K_matern_general(coord, [], ell, nu);

% filter K using difference filter
if ~isempty(L)
  K = L*(L*K)';
  K = (K+K')/2;
end
if strcmp(tag, 'func')
  K = @(x)(K*x);
end

% derivatives of K
for i = 1:length(dK)
  if ~isempty(L)
    dK{i} = L*(L*dK{i})';
    dK{i} = (dK{i}+dK{i}')/2;
  end
  if strcmp(tag, 'func')
    dK{i} = @(x)(dK{i}*x);
  end
end

% no preconditioner
M = [];
