function [L] = diffFilter1d(coord, order)
%function [L] = diffFilter1d(coord, order)
%
% This function computes a filtering matrix L for a set of points
% arbitrarily located in 1D. This function supports up to second order
% difference.
%
% INPUTS
%   coord: Coordinates of the *SORTED* points, including boundary.
%   order: Number of times filter is applied.
%
% OUTPUTS
%   L:     The filtering matrix. It is sparse and is retangular.
%

%Author: J. Chen
%$Date: 2011/11/23 11:46:41$

n = length(coord);
d = diff(coord);

if order == 0
  
  L = [];
  
elseif order == 1
  
  sd = 1./sqrt(d);
  L = spdiags([-sd sd], 0:1, n-1, n);

elseif order == 2
  
  sd = 1./sqrt(d(1:end-1)+d(2:end));
  sd1 = sd/2./d(1:end-1);
  sd2 = sd/2./d(2:end);
  L = spdiags([sd1 -sd1-sd2 sd2], 0:2, n-2, n);

else
  
  error('order must be <= 2 and >= 0');

end
