function [ftK] = lapFilterMat2d(tK, order)
%function [ftK] = lapFilterMat2d(tK, order)
%
% This function performs a Laplacian filtering on a 2-level Toeplitz
% matrix. The matrix is represented by its first column reshaped
% according to the size in each level. To perform the same filtering
% on a vector, use lapFilterVec2d.
%
% INPUTS
%   tK:    The matrix (first column, reshaped).
%   order: Number of times filter is applied.
%
% OUTPUTS
%   ftK:   The filtered result.
%

%Author: J. Chen
%$Date: 2011/11/23 11:47:10$

[m1, m2] = size(tK);

for iter = 1:order*2

  tK2 = zeros(m1+1,m2+1);
  tK2(2:m1+1,2:m2+1) = tK;
  
  tK2(1,:) = tK2(3,:);
  tK2(:,1) = tK2(:,3);

  tK = 4*tK2((2:m1)+0, (2:m2)+0) ...
        -tK2((2:m1)-1, (2:m2)+0) ...
        -tK2((2:m1)+1, (2:m2)+0) ...
        -tK2((2:m1)+0, (2:m2)-1) ...
        -tK2((2:m1)+0, (2:m2)+1);

  m1 = m1 - 1;
  m2 = m2 - 1;
  
end

ftK = tK;
