function [f] = lapFilterVec3d(t, order)
%function [f] = lapFilterVec3d(t, order)
%
% This function performs a Laplacian filtering on a vector. The vector
% is reshaped according to the underlying grid structure. To perform
% the same filtering on a 3-level Toeplitz matrix, use lapFilterMat3d.
%
% INPUTS
%   t:     The vector (reshaped).
%   order: Number of times filter is applied.
%
% OUTPUTS
%   f:     The filtered result.
%

%Author: J. Chen
%$Date: 2011/11/23 11:47:59$

[m1, m2, m3] = size(t);

for iter = 1:order

  t = 6*t((2:m1-1)+0, (2:m2-1)+0, (2:m3-1)+0) ...
       -t((2:m1-1)-1, (2:m2-1)+0, (2:m3-1)+0) ...
       -t((2:m1-1)+1, (2:m2-1)+0, (2:m3-1)+0) ...
       -t((2:m1-1)+0, (2:m2-1)-1, (2:m3-1)+0) ...
       -t((2:m1-1)+0, (2:m2-1)+1, (2:m3-1)+0) ...
       -t((2:m1-1)+0, (2:m2-1)+0, (2:m3-1)-1) ...
       -t((2:m1-1)+0, (2:m2-1)+0, (2:m3-1)+1);

  m1 = m1 - 2;
  m2 = m2 - 2;
  m3 = m3 - 2;
  
end

f = t;
