function [t] = get_intervals_geometric(min_eig, max_eig, success)
%function [t] = get_intervals_geometric(min_eig, max_eig, success)
%
% This function returns knots whose spacing is a geometric progression
% in [min_eig, max_eig]. This function is used by sampl_sqrt.
%
% INPUTS
%   min_eig: Left end of the interval.
%   max_eig: Right end of the interval.
%   success: Indicator where the spectrum estimate is accurate.
%
% OUTPUTS
%   t:       The knots.
%

%Author: J. Chen
%$Date: 2011/11/23 11:48:20$

r = 1.01;

if success
  l = min_eig/r;
else
  l = min_eig/2;
end

n = ceil( log(max_eig/l)/log(r) );
t = (r.^[0:n]')*l;
