function [y, inc] = sampl_embed(tK, maxinc)
%function [y, inc] = sampl_embed(tK, maxinc)
%
% This function computes a sample y from a Gaussian process with mean
% 0 and covariance K, by using the circulant embedding method. The
% covariance matrix must be a multilevel Toeplitz matrix. Up to
% three levels of Toeplitz structure is supported.
%
% INPUTS
%   tK:     The covariance matrix (first column, reshaped according to
%           the size in each level).
%   maxinc: Maximum number of embeddings to try.
%
% OUTPUTS
%   y:      Sample vector.
%   inc:    Actual nuber of embeddings tried.
%

%Author: J. Chen
%$Date: 2011/11/23 11:49:31$

dim = size(tK);
d = length(dim);
if d > 3
  error('Up to three levels of Toeplitz structure is supported.');
end

for inc = 1:maxinc
  [tK, eigval] = embed(tK);
  if isempty(find(eigval<0))
    break;
  else
    inc = inc + 1;
  end
end

if inc == maxinc + 1
  error('SAMPL_EMBED: unable to find embedding.');
end

y = ifftn( sqrt(eigval) .* fftn(randn(size(eigval))) );
if d == 1
  y = reshape(y(1:dim(1)), prod(dim), 1);
elseif d == 2
  y = reshape(y(1:dim(1), 1:dim(2)), prod(dim), 1);
elseif d == 3
  y = reshape(y(1:dim(1), 1:dim(2), 1:dim(3)), prod(dim), 1);
end
y = real(y);
