function [z] = three_term_recur(A, x, alpha, beta, gamma)
%function [z] = three_term_recur(A, x, alpha, beta, gamma)
%
% This function evaluates poly(A)*x, where the polynomial is the
% approximating polynomial computed by ls_poly, and represented by the
% Stieltjes coefficients alpha, beta and gamma. This function is used
% by sampl_sqrt.
%
% INPUTS
%   A: Matrix/Operator A.
%   x: Vector.
%   alpha, beta, gamma: Coefficients in the Stieltjes procedure.
%
% OUTPUTS
%   z: poly(A)*x, where poly approximates the function f input to ls_poly.
%

%Author: J. Chen
%$Date: 2011/11/23 11:49:56$

if isnumeric(A)
  A = @(x)(A*x);
end

v = x / beta(1);
z = gamma(1) * v;

for j = 1:length(alpha)
  if j ~= 1
    v1 = (A(v) - alpha(j)*v - beta(j)*v0) / beta(j+1);
  else
    v1 = (A(v) - alpha(j)*v) / beta(j+1);
  end
  z = z + gamma(j+1) * v1;
  
  v0 = v;
  v = v1;
end
