function [C, eigvalC] = embed(t)
%function [C, eigvalC] = embed(t)
%
% This function embeds a symmetric multilevel Toeplitz matrix T in a
% multilevel circulant matrix C. The matrix T cannot have more than 3
% levels of the Toeplitz structure.
%
% INPUTS
%   t:       First column of T, reshaped to the size of each level.
%
% OUTPUTS
%   C:       The circulant embedding (first column, reshaped).
%   eigvalC: Eigenvalues of the circulant embedding (reshaped).
%

%Author: J. Chen
%$Date: 2011/11/23 11:52:10$

if length(size(t)) > 3
  error('Number of Teoplitz levels must be <= 3');
end

dim = size(t);
d = length(dim);
C = zeros(dim*2);

if d == 1
  C(1:dim) = t;
  C(dim+2:2*dim) = C(dim:-1:2);
elseif d == 2
  C(1:dim(1), 1:dim(2)) = t;
  C(dim(1)+2:2*dim(1), :) = C(dim(1):-1:2, :);
  C(:, dim(2)+2:2*dim(2)) = C(:, dim(2):-1:2);
elseif d == 3
  C(1:dim(1), 1:dim(2), 1:dim(3)) = t;
  C(dim(1)+2:2*dim(1), :, :) = C(dim(1):-1:2, :, :);
  C(:, dim(2)+2:2*dim(2), :) = C(:, dim(2):-1:2, :);
  C(:, :, dim(3)+2:2*dim(3)) = C(:, :, dim(3):-1:2);
end

if nargout > 1
  eigvalC = fftn(C);
  eigvalC = real(eigvalC);
end
