function [y] = toeplitz_mult(eigvalC, x)
%function [y] = toeplitz_mult(eigvalC, x)
%
% This function mutiplies a symmetric multilevel Toeplitz matrix T by
% a vector. The vector can be a block vector.
%
% INPUTS
%   eigvalC: Eigenvalues of the circulant embedding of T, reshaped
%            to the size of each level. Call the function embed
%            before calling this function.
%   x:       (Block) vector.
%
% OUTPUTS
%   y:       T*x.
%

%Author: J. Chen
%$Date: 2011/11/23 11:52:31$

if length(size(eigvalC)) > 3
  error('Number of Teoplitz levels must be <= 3');
end

dim = size(eigvalC);
dim2 = dim/2;
d = length(dim);

y = zeros(size(x));
for i = 1:size(x,2)
  z = ifftn( eigvalC .* fftn( reshape(x(:,i),dim2), dim ) );
  if d == 1
    z = reshape(z(1:dim2(1)), prod(dim2), 1);
  elseif d == 2
    z = reshape(z(1:dim2(1), 1:dim2(2)), prod(dim2), 1);
  elseif d == 3
    z = reshape(z(1:dim2(1), 1:dim2(2), 1:dim2(3)), prod(dim2), 1);
  end
  y(:,i) = real(z);
end
