function [idx] = boundary3d(tetra)
%function [idx] = boundary3d(tetra)
%
% This function computes the boundary of a 3D mesh.
%
% INPUTS
%   tetra: Tetrahedrons of the mesh.
%
% OUTPUTS
%   idx:   Indicies of the boundary points.
%

%Author: J. Chen
%$Date: 2011/11/23 11:54:18$

H = zeros(size(tetra,1)*4,3);

cnt = 1;
for i = 1:size(tetra,1)
  idx = sort(tetra(i,:));
  H(cnt, :) = [idx(1) idx(2) idx(3)];
  cnt = cnt + 1;
  H(cnt, :) = [idx(1) idx(2) idx(4)];
  cnt = cnt + 1;
  H(cnt, :) = [idx(1) idx(3) idx(4)];
  cnt = cnt + 1;
  H(cnt, :) = [idx(2) idx(3) idx(4)];
  cnt = cnt + 1;
end

% non-duplicate rows of H
tag = zeros(size(H,1),1);
H = sortrows(H);
a = sum(abs(diff([zeros(1,3); H])),2);
b = sum(abs(diff([H; zeros(1,3)])),2);
idx = find(a~=0 & b~=0);

idx = unique([H(idx,1); H(idx,2); H(idx,3)]);
