function [eigvalC, C] = circ_precond(t)
%function [eigvalC, C] = circ_precond(t)
%
% This function computes the T.F. Chan's circulant preconditioner
% C for a multilevel Toeplitz matrix T.
%
% INPUTS
%   t:       First column of T, reshaped to the size of each level.
%
% OUTPUTS
%   eigvalC: Eigenvalues of the circulant preconditioner (reshaped).
%   C:       The circulant preconditioner (first column, reshaped).
%

%Author: J. Chen
%$Date: 2011/11/23 11:54:26$

d = length(size(t));

if d == 1

  [m1] = size(t);
  t2 = t;
  for i = 2:m1
    t2(i) = ( (m1-i+1)*t(i) + (i-1)*t(m1-i+2) ) / m1;
  end
  C = t2;

elseif d == 2
  
  [m1,m2] = size(t);
  t2 = t;
  for i = 2:m1
    t2(i,:) = ( (m1-i+1)*t(i,:) + (i-1)*t(m1-i+2,:) ) / m1;
  end
  t = t2;
  for i = 2:m2
    t2(:,i) = ( (m2-i+1)*t(:,i) + (i-1)*t(:,m2-i+2) ) / m2;
  end
  C = t2;

elseif d == 3
  
  [m1,m2,m3] = size(t);
  t2 = t;
  for i = 2:m1
    t2(i,:,:) = ( (m1-i+1)*t(i,:,:) + (i-1)*t(m1-i+2,:,:) ) / m1;
  end
  t = t2;
  for i = 2:m2
    t2(:,i,:) = ( (m2-i+1)*t(:,i,:) + (i-1)*t(:,m2-i+2,:) ) / m2;
  end
  t = t2;
  for i = 2:m3
    t2(:,:,i) = ( (m3-i+1)*t(:,:,i) + (i-1)*t(:,:,m3-i+2) ) / m3;
  end
  C = t2;

else
  
  error('Number of Toeplitz levels can only be 1, 2 or 3.');
  
end

eigvalC = fftn(C);
eigvalC = real(eigvalC);
