function [M] = stiff3d(coord, tetra, boundary)
%function [M] = stiff3d(coord, tetra, boundary)
%
% This function computes the stiffness matrix M for a 3D mesh.
%
% INPUTS
%   coord:    Coordinates of the points.
%   tetra:    Tetras of the mesh.
%   boundary: Indicies of the boundary points of the mesh.
%
% OUTPUTS
%   M:        Stiffness matrix, sparse.
%

%Author: J. Chen
%$Date: 2011/11/23 11:54:56$

n = size(coord,1);
A = zeros(size(tetra,1)*16, 3);

cnt = 1;
for i = 1:size(tetra,1)
  id = tetra(i,:);
  B = [coord(id(1),:)' coord(id(2),:)' coord(id(3),:)' coord(id(4),:)'; ...
       ones(1,4)];
  
  C = zeros(3,4);
  for j = 1:3
    for k = 1:4
      C(j,k) = (-1)^(j+k)*det(B([1:j-1 j+1:end],[1:k-1 k+1:end]));
    end
  end
  
  detB6 = abs(det(B))*6;
  
  for j = 1:4
    for k = 1:4
      A(cnt,:) = [id(j) id(k) C(:,j)'*C(:,k)/detB6];
      cnt = cnt + 1;
    end
  end
end

M = spconvert([A; n n 0]);
M(:,boundary) = [];
M = M';
M(:,boundary) = [];
M = M';
